@@ -29,7 +29,7 @@ import org.apache.commons.pool.ObjectPool;
  * @author Rodney Waldhoff
  * @author Glenn L. Nielsen
  * @author James House
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 758745 $ $Date: 2009-03-26 13:02:20 -0400 (Thu, 26 Mar 2009) $
  */
 public class PoolableConnection extends DelegatingConnection {
     /** The pool to which I should return. */
@@ -51,10 +51,8 @@ public class PoolableConnection extends DelegatingConnection {
      * @param conn my underlying connection
      * @param pool the pool to which I should return when closed
      * @param config the abandoned configuration settings
-     * @deprecated AbandonedConfig is now deprecated.
      */
-    public PoolableConnection(Connection conn, ObjectPool pool,
-                              AbandonedConfig config) {
+    public PoolableConnection(Connection conn, ObjectPool pool, AbandonedConfig config) {
         super(conn, config);
         _pool = pool;
     }
@@ -64,34 +62,56 @@ public class PoolableConnection extends DelegatingConnection {
      * Returns me to my pool.
      */
      public synchronized void close() throws SQLException {
-        boolean isClosed = false;
+        if (_closed) {
+            // already closed
+            return;
+        }
+
+        boolean isUnderlyingConectionClosed;
         try {
-            isClosed = isClosed();
+            isUnderlyingConectionClosed = _conn.isClosed();
         } catch (SQLException e) {
             try {
                 _pool.invalidateObject(this); // XXX should be guarded to happen at most once
+            } catch(IllegalStateException ise) {
+                // pool is closed, so close the connection
+                passivate();
+                getInnermostDelegate().close();
             } catch (Exception ie) {
                 // DO NOTHING the original exception will be rethrown
             }
-            throw new SQLNestedException("Cannot close connection (isClosed check failed)", e);
+            throw (SQLException) new SQLException("Cannot close connection (isClosed check failed)").initCause(e);
         }
-        if (isClosed) {
-            try {
-                _pool.invalidateObject(this); // XXX should be guarded to happen at most once
-            } catch (Exception ie) {
-                // DO NOTHING, "Already closed" exception thrown below
-            }
-            throw new SQLException("Already closed.");
-        } else {
+
+        if (!isUnderlyingConectionClosed) {
+            // Normal close: underlying connection is still open, so we
+            // simply need to return this proxy to the pool
             try {
                 _pool.returnObject(this); // XXX should be guarded to happen at most once
+            } catch(IllegalStateException e) {
+                // pool is closed, so close the connection
+                passivate();
+                getInnermostDelegate().close();
             } catch(SQLException e) {
                 throw e;
             } catch(RuntimeException e) {
                 throw e;
             } catch(Exception e) {
-                throw new SQLNestedException("Cannot close connection (return to pool failed)", e);
+                throw (SQLException) new SQLException("Cannot close connection (return to pool failed)").initCause(e);
             }
+        } else {
+            // Abnormal close: underlying connection closed unexpectedly, so we
+            // must destroy this proxy
+            try {
+                _pool.invalidateObject(this); // XXX should be guarded to happen at most once
+            } catch(IllegalStateException e) {
+                // pool is closed, so close the connection
+                passivate();
+                getInnermostDelegate().close();
+            } catch (Exception ie) {
+                // DO NOTHING, "Already closed" exception thrown below
+            }
+            throw new SQLException("Already closed.");
         }
     }
 
@@ -101,6 +121,5 @@ public class PoolableConnection extends DelegatingConnection {
     public void reallyClose() throws SQLException {
         super.close();
     }
-
 }
 
