@@ -36,7 +36,7 @@ import org.apache.commons.pool.KeyedObjectPool;
  * @author Glenn L. Nielsen
  * @author James House
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 745860 $ $Date: 2009-02-19 08:45:07 -0500 (Thu, 19 Feb 2009) $
  */
 public class PoolablePreparedStatement extends DelegatingPreparedStatement implements PreparedStatement {
     /**
@@ -49,6 +49,8 @@ public class PoolablePreparedStatement extends DelegatingPreparedStatement imple
      */
     protected Object _key = null;
 
+    private volatile boolean batchAdded = false;
+
     /**
      * Constructor
      * @param stmt my underlying {@link PreparedStatement}
@@ -68,13 +70,28 @@ public class PoolablePreparedStatement extends DelegatingPreparedStatement imple
         }
     }
 
+    /**
+     * Add batch.
+     */
+    public void addBatch() throws SQLException {
+        super.addBatch();
+        batchAdded = true;
+    }
+
+    /**
+     * Clear Batch.
+     */
+    public void clearBatch() throws SQLException {
+        batchAdded = false;
+        super.clearBatch();
+    }
+
     /**
      * Return me to my pool.
      */
     public void close() throws SQLException {
-        if(isClosed()) {
-            throw new SQLException("Already closed");
-        } else {
+        // calling close twice should have no effect
+        if (!isClosed()) {
             try {
                 _pool.returnObject(_key,this);
             } catch(SQLException e) {
@@ -113,6 +130,9 @@ public class PoolablePreparedStatement extends DelegatingPreparedStatement imple
             }
             clearTrace();
         }
+        if (batchAdded) {
+            clearBatch();
+        }
         
         super.passivate();
     }
