@@ -28,7 +28,7 @@ import junit.framework.TestSuite;
  * TestSuite for BasicDataSource with abandoned connection trace enabled
  * 
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 884205 $ $Date: 2009-11-25 12:25:48 -0500 (Wed, 25 Nov 2009) $
  */
 public class TestAbandonedBasicDataSource extends TestBasicDataSource {
     public TestAbandonedBasicDataSource(String testName) {
@@ -56,56 +56,39 @@ public class TestAbandonedBasicDataSource extends TestBasicDataSource {
 
     // ---------- Abandoned Test -----------
 
-    private void getConnection1() throws Exception {
-        System.err.println("BEGIN getConnection1()");
-        Connection conn = ds.getConnection();
-        System.err.println("conn: " + conn);
-        System.err.println("END getConnection1()");
-    }
-
-    private void getConnection2() throws Exception {
-        System.err.println("BEGIN getConnection2()");
-        Connection conn = ds.getConnection();
-        System.err.println("conn: " + conn);
-        System.err.println("END getConnection2()");
-    }
-
-    private void getConnection3() throws Exception {
-        System.err.println("BEGIN getConnection3()");
-        Connection conn = ds.getConnection();
-        System.err.println("conn: " + conn);
-        System.err.println("END getConnection3()");
-    }
-
     public void testAbandoned() throws Exception {
         // force abandoned
         ds.setRemoveAbandonedTimeout(0);
         ds.setMaxActive(1);
 
-        System.err.println("----------------------------------------");
-        getConnection1();
-        getConnection2();
-        getConnection3();
-        System.err.println("----------------------------------------");
+        for (int i = 0; i < 3; i++) {
+            assertNotNull(ds.getConnection());
+        }
     }
     
     public void testAbandonedClose() throws Exception {
         // force abandoned
         ds.setRemoveAbandonedTimeout(0);
         ds.setMaxActive(1);
+        ds.setAccessToUnderlyingConnectionAllowed(true);
 
         Connection conn1 = getConnection();
         assertNotNull(conn1);
         assertEquals(1, ds.getNumActive());
         
-        Connection conn2 = getConnection();        
+        Connection conn2 = getConnection();
+        // Attempt to borrow object triggers abandoned cleanup
+        // conn1 should be closed by the pool to make room
         assertNotNull(conn2);
         assertEquals(1, ds.getNumActive());
+        // Verify that conn1 is closed
+        assertTrue(((DelegatingConnection) conn1).getInnermostDelegate().isClosed());
         
-        try { conn2.close(); } catch (SQLException ex) { }
+        conn2.close();
         assertEquals(0, ds.getNumActive());
         
-        try { conn1.close(); } catch (SQLException ex) { }
+        // Second close on conn1 is OK as of dbcp 1.3
+        conn1.close();
         assertEquals(0, ds.getNumActive());
     }
 
@@ -135,4 +118,46 @@ public class TestAbandonedBasicDataSource extends TestBasicDataSource {
         try { conn1.close(); } catch (SQLException ex) { }
         assertEquals(0, ds.getNumActive());
     }
+    
+    /**
+     * Verify that lastUsed property is updated when a connection
+     * creates or prepares a statement
+     */
+    public void testlastUsed() throws Exception {
+        ds.setRemoveAbandonedTimeout(1);
+        ds.setMaxActive(2);
+        Connection conn1 = ds.getConnection();
+        Thread.sleep(500);
+        conn1.createStatement(); // Should reset lastUsed
+        Thread.sleep(800);
+        Connection conn2 = ds.getConnection(); // triggers abandoned cleanup
+        conn1.createStatement(); // Should still be OK
+        conn2.close();
+        Thread.sleep(500);
+        conn1.prepareStatement("SELECT 1 FROM DUAL"); // reset
+        Thread.sleep(800);
+        ds.getConnection(); // trigger abandoned cleanup again
+        conn1.createStatement();         
+    }
+    
+    /**
+     * Verify that lastUsed property is updated when a connection
+     * prepares a callable statement.
+     */
+    public void testlastUsedPrepareCall() throws Exception {
+        ds.setRemoveAbandonedTimeout(1);
+        ds.setMaxActive(2);
+        Connection conn1 = ds.getConnection();
+        Thread.sleep(500);
+        conn1.prepareCall("{call home}"); // Should reset lastUsed
+        Thread.sleep(800);
+        Connection conn2 = ds.getConnection(); // triggers abandoned cleanup
+        conn1.prepareCall("{call home}"); // Should still be OK
+        conn2.close();
+        Thread.sleep(500);
+        conn1.prepareCall("{call home}"); // reset
+        Thread.sleep(800);
+        ds.getConnection(); // trigger abandoned cleanup again
+        conn1.createStatement();         
+    }
 }
