@@ -20,6 +20,7 @@ package org.apache.commons.dbcp;
 import java.io.IOException;
 import java.sql.Connection;
 import java.sql.SQLException;
+import java.util.Arrays;
 
 import junit.framework.Test;
 import junit.framework.TestSuite;
@@ -28,7 +29,7 @@ import junit.framework.TestSuite;
  * TestSuite for BasicDataSource
  * 
  * @author Dirk Verbeeck
- * @version $Revision: 522304 $ $Date: 2007-03-25 10:21:23 -0700 (Sun, 25 Mar 2007) $
+ * @version $Revision: 895859 $ $Date: 2010-01-04 21:18:58 -0500 (Mon, 04 Jan 2010) $
  */
 public class TestBasicDataSource extends TestConnectionPool {
     public TestBasicDataSource(String testName) {
@@ -44,11 +45,11 @@ public class TestBasicDataSource extends TestConnectionPool {
     }
 
     protected BasicDataSource ds = null;
-    private static String CATALOG = "test catalog";
+    private static final String CATALOG = "test catalog";
 
     public void setUp() throws Exception {
         super.setUp();
-        ds = new BasicDataSource();
+        ds = createDataSource();
         ds.setDriverClassName("org.apache.commons.dbcp.TesterDriver");
         ds.setUrl("jdbc:apache:commons:testdriver");
         ds.setMaxActive(getMaxActive());
@@ -60,6 +61,11 @@ public class TestBasicDataSource extends TestConnectionPool {
         ds.setUsername("username");
         ds.setPassword("password");
         ds.setValidationQuery("SELECT DUMMY FROM DUAL");
+        ds.setConnectionInitSqls(Arrays.asList(new String[] { "SELECT 1", "SELECT 2"}));
+    }
+
+    protected BasicDataSource createDataSource() throws Exception {
+        return new BasicDataSource();
     }
 
     public void tearDown() throws Exception {
@@ -67,10 +73,110 @@ public class TestBasicDataSource extends TestConnectionPool {
         ds.close();
         ds = null;
     }
+
+    public void testClose() throws Exception {
+        ds.setAccessToUnderlyingConnectionAllowed(true);
+
+        // active connection is held open when ds is closed
+        Connection activeConnection = getConnection();
+        Connection rawActiveConnection = ((DelegatingConnection) activeConnection).getInnermostDelegate();
+        assertFalse(activeConnection.isClosed());
+        assertFalse(rawActiveConnection.isClosed());
+
+        // idle connection is in pool but closed
+        Connection idleConnection = getConnection();
+        Connection rawIdleConnection = ((DelegatingConnection) idleConnection).getInnermostDelegate();
+        assertFalse(idleConnection.isClosed());
+        assertFalse(rawIdleConnection.isClosed());
+
+        // idle wrapper should be closed but raw connection should be open
+        idleConnection.close();
+        assertTrue(idleConnection.isClosed());
+        assertFalse(rawIdleConnection.isClosed());
+
+        ds.close();
+
+        // raw idle connection should now be closed
+        assertTrue(rawIdleConnection.isClosed());
+
+        // active connection should still be open
+        assertFalse(activeConnection.isClosed());
+        assertFalse(rawActiveConnection.isClosed());
+
+        // now close the active connection
+        activeConnection.close();
+
+        // both wrapper and raw active connection should be closed
+        assertTrue(activeConnection.isClosed());
+        assertTrue(rawActiveConnection.isClosed());
+        
+        // Verify SQLException on getConnection after close
+        try {
+            getConnection();
+            fail("Expecting SQLException");
+        } catch (SQLException ex) {
+            // Expected
+        }
+        
+        // Redundant close is OK
+        ds.close();
+        
+    }
+
+    public void testSetProperties() throws Exception {
+        // normal
+        ds.setConnectionProperties("name1=value1;name2=value2;name3=value3");
+        assertEquals(3, ds.connectionProperties.size());
+        assertEquals("value1", ds.connectionProperties.getProperty("name1"));
+        assertEquals("value2", ds.connectionProperties.getProperty("name2"));
+        assertEquals("value3", ds.connectionProperties.getProperty("name3"));
+
+        // make sure all properties are replaced
+        ds.setConnectionProperties("name1=value1;name2=value2");
+        assertEquals(2, ds.connectionProperties.size());
+        assertEquals("value1", ds.connectionProperties.getProperty("name1"));
+        assertEquals("value2", ds.connectionProperties.getProperty("name2"));
+        assertFalse(ds.connectionProperties.containsKey("name3"));
+
+        // no value is empty string
+        ds.setConnectionProperties("name1=value1;name2");
+        assertEquals(2, ds.connectionProperties.size());
+        assertEquals("value1", ds.connectionProperties.getProperty("name1"));
+        assertEquals("", ds.connectionProperties.getProperty("name2"));
+
+        // no value (with equals) is empty string
+        ds.setConnectionProperties("name1=value1;name2=");
+        assertEquals(2, ds.connectionProperties.size());
+        assertEquals("value1", ds.connectionProperties.getProperty("name1"));
+        assertEquals("", ds.connectionProperties.getProperty("name2"));
+
+        // single value
+        ds.setConnectionProperties("name1=value1");
+        assertEquals(1, ds.connectionProperties.size());
+        assertEquals("value1", ds.connectionProperties.getProperty("name1"));
+
+        // single value with trailing ;
+        ds.setConnectionProperties("name1=value1;");
+        assertEquals(1, ds.connectionProperties.size());
+        assertEquals("value1", ds.connectionProperties.getProperty("name1"));
+
+        // single value wit no value
+        ds.setConnectionProperties("name1");
+        assertEquals(1, ds.connectionProperties.size());
+        assertEquals("", ds.connectionProperties.getProperty("name1"));
+
+        // null should throw a NullPointerException
+        try {
+            ds.setConnectionProperties(null);
+            fail("Expected NullPointerException");
+        } catch (NullPointerException e) {
+            // expected
+        }
+    }
     
     public void testTransactionIsolationBehavior() throws Exception {
         Connection conn = getConnection();
-        assertTrue(conn != null);
+        assertNotNull(conn);
         assertEquals(Connection.TRANSACTION_READ_COMMITTED, conn.getTransactionIsolation());
         conn.setTransactionIsolation(Connection.TRANSACTION_READ_UNCOMMITTED);
         conn.close();
@@ -87,7 +193,7 @@ public class TestBasicDataSource extends TestConnectionPool {
     }
 
     public void testPooling() throws Exception {
-        // this also needs access to the undelying connection
+        // this also needs access to the underlying connection
         ds.setAccessToUnderlyingConnectionAllowed(true);
         super.testPooling();
     }    
@@ -140,6 +246,66 @@ public class TestBasicDataSource extends TestConnectionPool {
             }
         }
     }
+    
+    public void testValidationQueryTimoutFail() {
+        ds.setTestOnBorrow(true);
+        ds.setValidationQueryTimeout(3); // Too fast for TesterStatement
+        try {
+            ds.getConnection();
+            fail("expected SQLException");
+        } catch (SQLException ex) {
+            if (ex.toString().indexOf("timeout") < 0) {
+                fail("expected timeout error message");
+            }
+        }
+    }
+    
+    public void testValidationQueryTimeoutZero() throws Exception {
+        ds.setTestOnBorrow(true);
+        ds.setTestOnReturn(true);
+        ds.setValidationQueryTimeout(0);
+        Connection con = ds.getConnection();
+        con.close();
+    }
+    
+    public void testValidationQueryTimeoutNegative() throws Exception {
+        ds.setTestOnBorrow(true);
+        ds.setTestOnReturn(true);
+        ds.setValidationQueryTimeout(-1);
+        Connection con = ds.getConnection();
+        con.close();
+    }
+    
+    public void testValidationQueryTimeoutSucceed() throws Exception {
+        ds.setTestOnBorrow(true);
+        ds.setTestOnReturn(true);
+        ds.setValidationQueryTimeout(100); // Works for TesterStatement
+        Connection con = ds.getConnection();
+        con.close();
+    }
+
+    public void testEmptyInitConnectionSql() throws Exception {
+        ds.setConnectionInitSqls(Arrays.asList(new String[]{"", "   "}));
+        assertNotNull(ds.getConnectionInitSqls());
+        assertEquals(0, ds.getConnectionInitSqls().size());
+
+        ds.setConnectionInitSqls(null);
+        assertNotNull(ds.getConnectionInitSqls());
+        assertEquals(0, ds.getConnectionInitSqls().size());
+    }
+
+    public void testInvalidConnectionInitSql() {
+        try {
+            ds.setConnectionInitSqls(Arrays.asList(new String[]{"SELECT 1","invalid"}));
+            ds.getConnection();
+            fail("expected SQLException");
+        }
+        catch (SQLException e) {
+            if (e.toString().indexOf("invalid") < 0) {
+                fail("expected detailed error message");
+            }
+        }
+    }
 
     public void testSetValidationTestProperties() {
         // defaults
@@ -247,7 +413,7 @@ public class TestBasicDataSource extends TestConnectionPool {
             fail("Expected SQLException");
         }
         catch(SQLException ex) { }
-        
+
         assertEquals(0, ds.getNumActive());
     }
     
@@ -308,7 +474,7 @@ public class TestBasicDataSource extends TestConnectionPool {
     public void testCreateDataSourceCleanupThreads() throws Exception {
         ds.close();
         ds = null;
-        ds = new BasicDataSource();
+        ds = createDataSource();
         ds.setDriverClassName("org.apache.commons.dbcp.TesterDriver");
         ds.setUrl("jdbc:apache:commons:testdriver");
         ds.setMaxActive(getMaxActive());
@@ -326,7 +492,7 @@ public class TestBasicDataSource extends TestConnectionPool {
         int threadCount = Thread.activeCount();
         for (int i = 0; i < 10; i++) {
             try {
-                Connection con = ds.getConnection();
+                ds.getConnection();
             } catch (SQLException ex) {
                 // ignore
             }
