@@ -27,7 +27,7 @@ import junit.framework.TestSuite;
 /**
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 896720 $ $Date: 2010-01-06 18:43:07 -0500 (Wed, 06 Jan 2010) $
  */
 public class TestDelegatingPreparedStatement extends TestCase {
     public TestDelegatingPreparedStatement(String testName) {
@@ -73,20 +73,49 @@ public class TestDelegatingPreparedStatement extends TestCase {
     
     public void testHashCode() {
         delegateStmt = new TesterPreparedStatement(delegateConn,"select * from foo");
-        DelegatingPreparedStatement stmt = new DelegatingPreparedStatement(conn,delegateStmt);
+        DelegatingPreparedStatement stmt1 = new DelegatingPreparedStatement(conn,delegateStmt);
         DelegatingPreparedStatement stmt2 = new DelegatingPreparedStatement(conn,delegateStmt);
-        assertEquals(stmt.hashCode(), stmt2.hashCode());
+        assertEquals(stmt1.hashCode(), stmt2.hashCode());
     }
     
     public void testEquals() {
         delegateStmt = new TesterPreparedStatement(delegateConn,"select * from foo");
-        DelegatingPreparedStatement stmt = new DelegatingPreparedStatement(conn, delegateStmt);
+        PreparedStatement del = new TesterPreparedStatement(delegateConn,"select * from foo");
+        DelegatingPreparedStatement stmt1 = new DelegatingPreparedStatement(conn, delegateStmt);
         DelegatingPreparedStatement stmt2 = new DelegatingPreparedStatement(conn, delegateStmt);
         DelegatingPreparedStatement stmt3 = new DelegatingPreparedStatement(conn, null);
+        DelegatingPreparedStatement stmt4 = new DelegatingPreparedStatement(conn, del);
         
-        assertTrue(!stmt.equals(null));
-        assertTrue(stmt.equals(stmt2));
-        assertTrue(!stmt.equals(stmt3));
+        // Nothing is equal to null
+        assertFalse(stmt1.equals(null));
+        assertFalse(stmt2.equals(null));
+        assertFalse(stmt3.equals(null));
+        assertFalse(stmt4.equals(null));
+        
+        // 1 & 2 are equivalent
+        assertTrue(stmt1.equals(stmt2));
+        assertTrue(stmt2.equals(stmt1)); // reflexive
+
+        // 1 & 3 are not (different statements, one null)
+        assertFalse(stmt1.equals(stmt3));
+        assertFalse(stmt3.equals(stmt1)); // reflexive
+
+        // 1 & 4 are not (different statements)
+        assertFalse(stmt1.equals(stmt4));
+        assertFalse(stmt4.equals(stmt1)); // reflexive
+
+        // Check self-equals
+        assertTrue(stmt1.equals(stmt1));
+        assertTrue(stmt2.equals(stmt2));
+        assertFalse(stmt3.equals(stmt3)); // because underlying statement is null
+        assertTrue(stmt4.equals(stmt4));
+        
+        DelegatingStatement dstmt1 = stmt1;
+        
+        // 1 & 2 are equivalent
+        assertTrue(dstmt1.equals(stmt2));
+        assertTrue(stmt2.equals(dstmt1)); // reflexive
+
     }
 
 }
