@@ -17,6 +17,7 @@
 
 package org.apache.commons.dbcp;
 
+import java.io.ByteArrayOutputStream;
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.sql.Connection;
@@ -36,7 +37,7 @@ import org.apache.commons.pool.impl.GenericObjectPool;
  * based {@link PoolingDriver}.
  * @author Rodney Waldhoff
  * @author Sean C. Sullivan
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 896526 $ $Date: 2010-01-06 11:51:06 -0500 (Wed, 06 Jan 2010) $
  */
 public class TestManual extends TestConnectionPool {
     public TestManual(String testName) {
@@ -63,47 +64,50 @@ public class TestManual extends TestConnectionPool {
         driver = new PoolingDriver();
         driver.registerPool("test",pool);
         PoolingDriver.setAccessToUnderlyingConnectionAllowed(true);
-        DriverManager.registerDriver(driver);
     }
 
     public void tearDown() throws Exception {
         driver.closePool("test");
-        DriverManager.deregisterDriver(driver);
+        super.tearDown();
     }
     
     public void test1() {
         GenericObjectPool connectionPool = new GenericObjectPool(null);
         ConnectionFactory connectionFactory = new DriverManagerConnectionFactory("jdbc:some:connect:string","username","password");
-        PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(connectionFactory,connectionPool,null,null,false,true);
-        PoolingDataSource dataSource = new PoolingDataSource(connectionPool);
+        new PoolableConnectionFactory(connectionFactory,connectionPool,null,null,false,true);
+        new PoolingDataSource(connectionPool);
     }
 
     public void test2() {
         GenericObjectPool connectionPool = new GenericObjectPool(null);
         ConnectionFactory connectionFactory = new DriverManagerConnectionFactory("jdbc:some:connect:string","username","password");
-        PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(connectionFactory,connectionPool,null,null,false,true);
-        PoolingDriver driver = new PoolingDriver();
-        driver.registerPool("example",connectionPool);
+        new PoolableConnectionFactory(connectionFactory,connectionPool,null,null,false,true);
+        PoolingDriver driver2 = new PoolingDriver();
+        driver2.registerPool("example",connectionPool);
     }
 
 
-    /** @see http://issues.apache.org/bugzilla/show_bug.cgi?id=28912 */
+    /** @see "http://issues.apache.org/bugzilla/show_bug.cgi?id=28912" */
     public void testReportedBug28912() throws Exception {
         Connection conn1 = getConnection();
         assertNotNull(conn1);
+        assertFalse(conn1.isClosed());
         conn1.close();        
-        
+
         Connection conn2 = getConnection();
         assertNotNull(conn2);
-        
-        try {
-            conn1.close();
-            fail("Expected SQLException");
-        }
-        catch (SQLException e) { }
+
+        assertTrue(conn1.isClosed());
+        assertFalse(conn2.isClosed());
+
+        // should be able to call close multiple times with no effect
+        conn1.close();
+
+        assertTrue(conn1.isClosed());
+        assertFalse(conn2.isClosed());
     }
     
-    /** @see http://issues.apache.org/bugzilla/show_bug.cgi?id=12400 */
+    /** @see "http://issues.apache.org/bugzilla/show_bug.cgi?id=12400" */
     public void testReportedBug12400() throws Exception {
         ObjectPool connectionPool = new GenericObjectPool(
             null,
@@ -124,8 +128,8 @@ public class TestManual extends TestConnectionPool {
                 false,
                 true);
         assertNotNull(poolableConnectionFactory);
-        PoolingDriver driver = new PoolingDriver();
-        driver.registerPool("neusoftim",connectionPool);
+        PoolingDriver driver2 = new PoolingDriver();
+        driver2.registerPool("neusoftim",connectionPool);
         Connection[] conn = new Connection[25];
         for(int i=0;i<25;i++) {
             conn[i] = DriverManager.getConnection("jdbc:apache:commons:dbcp:neusoftim");
@@ -144,11 +148,11 @@ public class TestManual extends TestConnectionPool {
         assertNotNull(conn);
         conn.close();
         
-        PoolingDriver driver = (PoolingDriver) DriverManager.getDriver("jdbc:apache:commons:dbcp:");
-        driver.closePool("test");
+        PoolingDriver driver2 = (PoolingDriver) DriverManager.getDriver("jdbc:apache:commons:dbcp:");
+        driver2.closePool("test");
 
         try {
-            conn = DriverManager.getConnection("jdbc:apache:commons:dbcp:test");
+            DriverManager.getConnection("jdbc:apache:commons:dbcp:test");
             fail("expected SQLException");
         }
         catch (SQLException e) {
@@ -164,8 +168,8 @@ public class TestManual extends TestConnectionPool {
         assertEquals(1, pool.getNumActive());
         assertEquals(0, pool.getNumIdle());
 
-        PoolingDriver driver = (PoolingDriver) DriverManager.getDriver("jdbc:apache:commons:dbcp:");
-        driver.invalidateConnection(conn);
+        PoolingDriver driver2 = (PoolingDriver) DriverManager.getDriver("jdbc:apache:commons:dbcp:");
+        driver2.invalidateConnection(conn);
 
         assertEquals(0, pool.getNumActive());
         assertEquals(0, pool.getNumIdle());
@@ -173,8 +177,9 @@ public class TestManual extends TestConnectionPool {
     }
 
     public void testLogWriter() throws Exception {
-        PrintStream ps = System.out;
-        PrintWriter pw = new PrintWriter(System.err);
+        PrintStream ps = new PrintStream(new ByteArrayOutputStream());
+        PrintWriter pw = new PrintWriter(new ByteArrayOutputStream());
+        System.setErr(new PrintStream(new ByteArrayOutputStream()));
         SQLException ex;
         
         DriverManager.setLogWriter(pw);
