@@ -25,19 +25,29 @@ import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.sql.Statement;
 import java.util.Map;
+/* JDBC_4_ANT_KEY_BEGIN */
+import java.sql.Array;
+import java.sql.Blob;
+import java.sql.Clob;
+import java.sql.NClob;
+import java.sql.SQLClientInfoException;
+import java.sql.SQLXML;
+import java.sql.Struct;
+import java.util.Properties;
+/* JDBC_4_ANT_KEY_END */
 
 /**
  * A dummy {@link Connection}, for testing purposes.
  * 
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 895744 $ $Date: 2010-01-04 13:48:05 -0500 (Mon, 04 Jan 2010) $
  */
 public class TesterConnection implements Connection {
     protected boolean _open = true;
     protected boolean _autoCommit = true;
     protected int _transactionIsolation = 1;
-    protected DatabaseMetaData _metaData = null;
+    protected DatabaseMetaData _metaData = new TesterDatabaseMetaData();
     protected String _catalog = null;
     protected Map _typeMap = null;
     protected boolean _readOnly = false;
@@ -65,7 +75,7 @@ public class TesterConnection implements Connection {
     }
 
     public void close() throws SQLException {
-        checkOpen();
+        checkFailure();
         _open = false;
     }
 
@@ -136,12 +146,12 @@ public class TesterConnection implements Connection {
         if ("warning".equals(sql)) {
             setWarnings(new SQLWarning("warning in prepareCall"));
         }
-        return null;
+        return new TesterCallableStatement(this);
     }
 
     public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
         checkOpen();
-        return null;
+        return new TesterCallableStatement(this);
     }
 
     public PreparedStatement prepareStatement(String sql) throws SQLException {
@@ -202,7 +212,7 @@ public class TesterConnection implements Connection {
     
     protected void checkFailure() throws SQLException {
         if (failure != null) {
-            throw new SQLNestedException("TesterConnection failure", failure);
+            throw (SQLException) new SQLException("TesterConnection failure").initCause(failure);
         }
     }
     
@@ -210,11 +220,6 @@ public class TesterConnection implements Connection {
         this.failure = failure;
     }
     
-    // ------------------- JDBC 3.0 -----------------------------------------
-    // Will be commented by the build process on a JDBC 2.0 system
-
-/* JDBC_3_ANT_KEY_BEGIN */
-
     public int getHoldability() throws SQLException {
         throw new SQLException("Not implemented.");
     }
@@ -243,37 +248,90 @@ public class TesterConnection implements Connection {
                                      int resultSetConcurrency,
                                      int resultSetHoldability)
         throws SQLException {
-        throw new SQLException("Not implemented.");
+        return createStatement();
     }
 
     public PreparedStatement prepareStatement(String sql, int resultSetType,
                                               int resultSetConcurrency,
                                               int resultSetHoldability)
         throws SQLException {
-        throw new SQLException("Not implemented.");
+        return prepareStatement(sql);
     }
 
     public CallableStatement prepareCall(String sql, int resultSetType,
                                          int resultSetConcurrency,
                                          int resultSetHoldability)
         throws SQLException {
-        throw new SQLException("Not implemented.");
+        return prepareCall(sql);
     }
 
     public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
         throws SQLException {
-        throw new SQLException("Not implemented.");
+        return prepareStatement(sql);
     }
 
     public PreparedStatement prepareStatement(String sql, int columnIndexes[])
         throws SQLException {
-        throw new SQLException("Not implemented.");
+        return prepareStatement(sql);
     }
 
     public PreparedStatement prepareStatement(String sql, String columnNames[])
         throws SQLException {
+        return prepareStatement(sql);
+    }
+
+/* JDBC_4_ANT_KEY_BEGIN */
+
+    public boolean isWrapperFor(Class<?> iface) throws SQLException {
         throw new SQLException("Not implemented.");
     }
 
-/* JDBC_3_ANT_KEY_END */
+    public <T> T unwrap(Class<T> iface) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public Blob createBlob() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public Clob createClob() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public NClob createNClob() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public SQLXML createSQLXML() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public boolean isValid(int timeout) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public void setClientInfo(String name, String value) throws SQLClientInfoException {
+        throw new SQLClientInfoException();
+    }
+
+    public void setClientInfo(Properties properties) throws SQLClientInfoException {
+        throw new SQLClientInfoException();
+    }
+
+    public Properties getClientInfo() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public String getClientInfo(String name) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+/* JDBC_4_ANT_KEY_END */
 }
