@@ -39,10 +39,10 @@ import java.util.Properties;
  * 
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 907288 $ $Date: 2010-02-06 14:42:58 -0500 (Sat, 06 Feb 2010) $
  */
 public class TesterDriver implements Driver {
-    private static Properties validUserPasswords = new Properties();
+    private static final Properties validUserPasswords = new Properties();
     static {
         try {
             DriverManager.registerDriver(new TesterDriver());
@@ -58,7 +58,9 @@ public class TesterDriver implements Driver {
      * TesterDriver specific method to add users to the list of valid users 
      */
     public static void addUser(String username, String password) {
-        validUserPasswords.put(username, password);
+        synchronized (validUserPasswords) {
+            validUserPasswords.put(username, password);
+        }
     }
 
     public boolean acceptsURL(String url) throws SQLException {
@@ -67,17 +69,15 @@ public class TesterDriver implements Driver {
 
     private void assertValidUserPassword(String user, String password) 
         throws SQLException {
-        String realPassword = validUserPasswords.getProperty(user);
-        if (realPassword == null) 
-        {
-            throw new SQLException(user + " is not a valid username.");
-        }
-        if (!realPassword.equals(password)) 
-        {
-            throw new SQLException(password + 
-                                   " is not the correct password for " +
-                                   user + ".  The correct password is " +
-                                   realPassword);
+        synchronized (validUserPasswords) {
+            String realPassword = validUserPasswords.getProperty(user);
+            if (realPassword == null) {
+                throw new SQLException(user + " is not a valid username.");
+            }
+            if (!realPassword.equals(password)) {
+                throw new SQLException(password + " is not the correct password for " + user
+                        + ".  The correct password is " + realPassword);
+            }
         }
     }
 
@@ -116,10 +116,10 @@ public class TesterDriver implements Driver {
         return new DriverPropertyInfo[0];
     }
 
-    protected static String CONNECT_STRING = "jdbc:apache:commons:testdriver";
+    protected static final String CONNECT_STRING = "jdbc:apache:commons:testdriver";
 
     // version numbers
-    protected static int MAJOR_VERSION = 1;
-    protected static int MINOR_VERSION = 0;
+    protected static final int MAJOR_VERSION = 1;
+    protected static final int MINOR_VERSION = 0;
 
 }
