@@ -28,7 +28,7 @@ import java.sql.Statement;
  * 
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 883942 $ $Date: 2009-11-24 20:00:03 -0500 (Tue, 24 Nov 2009) $
  */
 public class TesterStatement implements Statement {
     public TesterStatement(Connection conn) {
@@ -60,15 +60,22 @@ public class TesterStatement implements Statement {
         checkOpen();
         if("null".equals(sql)) {
             return null;
-        } if("invalid".equals(sql)) {
+        } 
+        if("invalid".equals(sql)) {
             throw new SQLException("invalid query");
-        } if ("broken".equals(sql)) {
+        }
+        if ("broken".equals(sql)) {
             throw new SQLException("broken connection");
-        } if("select username".equals(sql)) {
+        }  
+        if("select username".equals(sql)) {
             String username = ((TesterConnection) _connection).getUsername();
             Object[][] data = {{username}};
             return new TesterResultSet(this, data);
         } else {
+            // Simulate timeout if queryTimout is set to less than 5 seconds
+            if (_queryTimeout > 0 && _queryTimeout < 5) { 
+                throw new SQLException("query timeout");
+            }
             return new TesterResultSet(this);
         }
     }
@@ -79,7 +86,11 @@ public class TesterStatement implements Statement {
     }
 
     public void close() throws SQLException {
-        checkOpen();
+        // calling close twice has no effect
+        if (!_open) {
+            return;
+        }
+
         _open = false;
         if (_resultSet != null) {
             _resultSet.close();
@@ -142,6 +153,9 @@ public class TesterStatement implements Statement {
 
     public boolean execute(String sql) throws SQLException {
         checkOpen();
+        if("invalid".equals(sql)) {
+            throw new SQLException("invalid query");
+        }
         return _executeResponse;
     }
 
@@ -217,16 +231,12 @@ public class TesterStatement implements Statement {
         }
     }
 
-    // ------------------- JDBC 3.0 -----------------------------------------
-    // Will be commented by the build process on a JDBC 2.0 system
-
-/* JDBC_3_ANT_KEY_BEGIN */
     public boolean getMoreResults(int current) throws SQLException {
         throw new SQLException("Not implemented.");
     }
 
     public ResultSet getGeneratedKeys() throws SQLException {
-        throw new SQLException("Not implemented.");
+        return new TesterResultSet(this);
     }
 
     public int executeUpdate(String sql, int autoGeneratedKeys)
@@ -263,6 +273,27 @@ public class TesterStatement implements Statement {
         checkOpen();
         throw new SQLException("Not implemented.");
     }
-/* JDBC_3_ANT_KEY_END */
 
+/* JDBC_4_ANT_KEY_BEGIN */
+
+    public boolean isWrapperFor(Class<?> iface) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public <T> T unwrap(Class<T> iface) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public boolean isClosed() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public void setPoolable(boolean poolable) throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+
+    public boolean isPoolable() throws SQLException {
+        throw new SQLException("Not implemented.");
+    }
+/* JDBC_4_ANT_KEY_END */
 }
