@@ -28,12 +28,13 @@ import junit.framework.Test;
 import junit.framework.TestSuite;
 
 import org.apache.commons.dbcp.TestConnectionPool;
+import org.apache.commons.dbcp.TesterDriver;
 import org.apache.commons.dbcp.cpdsadapter.DriverAdapterCPDS;
 
 /**
  * @author John McNally
  * @author Dirk Verbeeck
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 907359 $ $Date: 2010-02-06 20:07:29 -0500 (Sat, 06 Feb 2010) $
  */
 public class TestSharedPoolDataSource extends TestConnectionPool {
     public TestSharedPoolDataSource(String testName) {
@@ -52,12 +53,14 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     private DataSource ds;
 
     public void setUp() throws Exception {
+        super.setUp();
         pcds = new DriverAdapterCPDS();
         pcds.setDriver("org.apache.commons.dbcp.TesterDriver");
         pcds.setUrl("jdbc:apache:commons:testdriver");
         pcds.setUser("foo");
         pcds.setPassword("bar");
         pcds.setPoolPreparedStatements(false);
+        pcds.setAccessToUnderlyingConnectionAllowed(true);
 
         SharedPoolDataSource tds = new SharedPoolDataSource();
         tds.setConnectionPoolDataSource(pcds);
@@ -69,16 +72,26 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         ds = tds;
     }
 
+
+    public void testBackPointers() throws Exception {
+        // todo disabled until a wrapping issuen in SharedPoolDataSource are resolved
+    }
+
     /**
      * Switching 'u1 -> 'u2' and 'p1' -> 'p2' will
      * exhibit the bug detailed in 
      * http://issues.apache.org/bugzilla/show_bug.cgi?id=18905
+     * 
+     * Starting with a successful connection, then incorrect password,
+     * then correct password for same user illustrates
+     * JIRA: DBCP-245
      */
     public void testIncorrectPassword() throws Exception 
     {
+        ds.getConnection("u2", "p2").close();
         try {
             // Use bad password
-            ds.getConnection("u1", "zlsafjk").close();
+            ds.getConnection("u1", "zlsafjk");
             fail("Able to retrieve connection with incorrect password");
         } catch (SQLException e1) {
             // should fail
@@ -89,7 +102,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         ds.getConnection("u1", "p1").close();
         try 
         {
-            ds.getConnection("u1", "x").close();
+            ds.getConnection("u1", "x");
             fail("Able to retrieve connection with incorrect password");
         }
         catch (SQLException e)
@@ -103,17 +116,30 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         
         // Make sure we can still use our good password.
         ds.getConnection("u1", "p1").close();
+        
+        // Try related users and passwords
+        ds.getConnection("foo", "bar").close();
+        try {
+            ds.getConnection("foob", "ar");
+            fail("Should have caused an SQLException");
+        } catch (SQLException expected) {
+        }
+        try {
+            ds.getConnection("foo", "baz");
+            fail("Should have generated SQLException");
+        } catch (SQLException expected) {
+        }
     }
 
 
     public void testSimple() throws Exception 
     {
         Connection conn = ds.getConnection();
-        assertTrue(null != conn);
+        assertNotNull(conn);
         PreparedStatement stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         ResultSet rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
@@ -123,11 +149,11 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     public void testSimpleWithUsername() throws Exception 
     {
         Connection conn = ds.getConnection("u1", "p1");
-        assertTrue(null != conn);
+        assertNotNull(conn);
         PreparedStatement stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         ResultSet rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
@@ -170,21 +196,21 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         throws Exception 
     {
         Connection conn = ds.getConnection();
-        assertTrue(null != conn);
+        assertNotNull(conn);
 
         PreparedStatement stmt = 
             conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         ResultSet rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
         
         stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
@@ -201,20 +227,20 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         }
 
         conn = ds.getConnection();
-        assertTrue(null != conn);
+        assertNotNull(conn);
 
         stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
 
         stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
@@ -306,109 +332,67 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         }
     }
     
-    public void testMultipleThreads() throws Exception {
-        assertTrue(multipleThreads(1));
-        assertTrue(!multipleThreads(2 * (int)(getMaxWait())));
+    public void testMaxWait() throws Exception {
+        final int maxWait = 1000;
+        final int theadCount = 20;
+        
+        ((SharedPoolDataSource)ds).setMaxWait(maxWait);
+        // Obtain all the connections from the pool
+        Connection[] c = new Connection[getMaxActive()];
+        for (int i=0; i<c.length; i++) {
+            c[i] = ds.getConnection("foo","bar");
+            assertTrue(c[i] != null);            
+        }
+
+        long start = System.currentTimeMillis();
+        
+        // Run a thread test with minimal hold time
+        // All threads should end after maxWait - DBCP-291
+        final PoolTest[] pts = new PoolTest[theadCount];
+        ThreadGroup threadGroup = new ThreadGroup("testMaxWait");
+
+        // Should take ~maxWait for threads to stop 
+        for (int i = 0; i < pts.length; i++) {
+            (pts[i] = new PoolTest(threadGroup, 1, true)).start();
+        }
+        
+        // Wait for all the threads to complete
+        for (int i = 0; i < pts.length; i++) {
+            final PoolTest poolTest = pts[i];
+            poolTest.getThread().join();
+        }
+
+        
+        long end = System.currentTimeMillis();
+        
+        System.out.println("testMaxWait took " + (end-start) + " ms. Maxwait: "+maxWait);
+        
+        // Threads should time out in parallel - allow double that to be safe
+        assertTrue((end-start) < (2 * maxWait)); 
+
+        // Put all the connections back in the pool
+        for (int i=0; i<c.length; i++) {
+            c[i].close();
+        }
     }
 
-    private boolean multipleThreads(int holdTime) throws Exception {
-        long startTime = System.currentTimeMillis();
-        final boolean[] success = new boolean[1];
-        success[0] = true;
-        final PoolTest[] pts = new PoolTest[2 * getMaxActive()];
-        ThreadGroup threadGroup = new ThreadGroup("foo") {
-            public void uncaughtException(Thread t, Throwable e) {
-                /*
-                for (int i = 0; i < pts.length; i++)
-                {
-                    System.out.println(i + ": " + pts[i].reportState());
-                }
-                */
-                for (int i = 0; i < pts.length; i++) {
-                    pts[i].stop();
-                }
-
-                //e.printStackTrace();
-                success[0] = false;
-            }
-        };
-
-        for (int i = 0; i < pts.length; i++) {
-            pts[i] = new PoolTest(threadGroup, holdTime);
-        }
-        Thread.sleep(10 * holdTime);
-        for (int i = 0; i < pts.length; i++) {
-            pts[i].stop();
-        }
-        long time = System.currentTimeMillis() - startTime;
-        // - (pts.length*10*holdTime);
-        System.out.println("Multithread test time = " + time + " ms");
-
-        Thread.sleep(holdTime);
-        return success[0];
+    public void testMultipleThreads1() throws Exception {
+        // Override wait time in order to allow for Thread.sleep(1) sometimes taking a lot longer on
+        // some JVMs, e.g. Windows.
+        final int defaultMaxWait = 430;
+        ((SharedPoolDataSource) ds).setMaxWait(defaultMaxWait);
+        multipleThreads(1, false, false, defaultMaxWait);
     }
 
-    private static int currentThreadCount = 0;
-
-    private class PoolTest implements Runnable {
-        /**
-         * The number of milliseconds to hold onto a database connection
-         */
-        private int connHoldTime;
-
-        private boolean isRun;
-
-        private String state;
-
-        protected PoolTest(ThreadGroup threadGroup, int connHoldTime) {
-            this.connHoldTime = connHoldTime;
-            Thread thread =
-                new Thread(threadGroup, this, "Thread+" + currentThreadCount++);
-            thread.setDaemon(false);
-            thread.start();
-        }
-
-        public void run() {
-            isRun = true;
-            while (isRun) {
-                try {
-                    Connection conn = null;
-                    state = "Getting Connection";
-                    conn = getConnection();
-                    state = "Using Connection";
-                    assertTrue(null != conn);
-                    PreparedStatement stmt =
-                        conn.prepareStatement("select * from dual");
-                    assertTrue(null != stmt);
-                    ResultSet rset = stmt.executeQuery();
-                    assertTrue(null != rset);
-                    assertTrue(rset.next());
-                    state = "Holding Connection";
-                    Thread.sleep(connHoldTime);
-                    state = "Returning Connection";
-                    rset.close();
-                    stmt.close();
-                    conn.close();
-                } catch (RuntimeException e) {
-                    throw e;
-                } catch (Exception e) {
-                    throw new RuntimeException(e.toString());
-                }
-            }
-        }
-
-        public void stop() {
-            isRun = false;
-        }
-
-        public String reportState() {
-            return state;
-        }
+    public void testMultipleThreads2() throws Exception {
+        final int defaultMaxWait = 500;
+        ((SharedPoolDataSource) ds).setMaxWait(defaultMaxWait);
+        multipleThreads(2 * defaultMaxWait, true, true, defaultMaxWait);
     }
 
     public void testTransactionIsolationBehavior() throws Exception {
         Connection conn = getConnection();
-        assertTrue(conn != null);
+        assertNotNull(conn);
         assertEquals(Connection.TRANSACTION_READ_COMMITTED, 
                      conn.getTransactionIsolation());
         conn.setTransactionIsolation(Connection.TRANSACTION_READ_UNCOMMITTED);
@@ -432,18 +416,58 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         pcds.setPoolPreparedStatements(true);
 
         Connection conn = ds.getConnection();
-        assertTrue(null != conn);
+        assertNotNull(conn);
         PreparedStatement stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        assertNotNull(stmt);
         ResultSet rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
         conn.close();
     }
 
-    public void testPoolPreparedStatements() throws Exception {
+    // There are 6 different prepareStatement statement methods so add a little
+    // complexity to reduce what would otherwise be lots of copy and paste
+    private static abstract class PrepareStatementCallback {
+        protected Connection conn;
+        void setConnection(Connection conn) {
+            this.conn = conn;
+        }
+        abstract PreparedStatement getPreparedStatement() throws SQLException;
+    }
+    private static class PscbString extends PrepareStatementCallback {
+        PreparedStatement getPreparedStatement() throws SQLException {
+            return conn.prepareStatement("select * from dual");
+        }
+    }
+    private static class PscbStringIntInt extends PrepareStatementCallback {
+        PreparedStatement getPreparedStatement() throws SQLException {
+            return conn.prepareStatement("select * from dual",0,0);
+        }
+    }
+    private static class PscbStringInt extends PrepareStatementCallback {
+        PreparedStatement getPreparedStatement() throws SQLException {
+            return conn.prepareStatement("select * from dual",0);
+        }
+    }
+    private static class PscbStringIntArray extends PrepareStatementCallback {
+        PreparedStatement getPreparedStatement() throws SQLException {
+            return conn.prepareStatement("select * from dual", new int[0]);
+        }
+    }
+    private static class PscbStringStringArray extends PrepareStatementCallback {
+        PreparedStatement getPreparedStatement() throws SQLException {
+            return conn.prepareStatement("select * from dual",new String[0]);
+        }
+    }
+    private static class PscbStringIntIntInt extends PrepareStatementCallback {
+        PreparedStatement getPreparedStatement() throws SQLException {
+            return conn.prepareStatement("select * from dual",0,0,0);
+        }
+    }
+    private void doTestPoolPreparedStatements(PrepareStatementCallback callBack)
+    throws Exception {
         DriverAdapterCPDS mypcds = new DriverAdapterCPDS();
         DataSource myds = null;
         mypcds.setDriver("org.apache.commons.dbcp.TesterDriver");
@@ -463,25 +487,26 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         myds = tds;
 
         Connection conn = ds.getConnection();
+        callBack.setConnection(conn);
         PreparedStatement stmt = null;
         ResultSet rset = null;
 
-        assertTrue(null != conn);
+        assertNotNull(conn);
 
-        stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        stmt = callBack.getPreparedStatement();
+        assertNotNull(stmt);
         long l1HashCode = stmt.hashCode();
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
 
-        stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        stmt = callBack.getPreparedStatement();
+        assertNotNull(stmt);
         long l2HashCode = stmt.hashCode();
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
@@ -492,21 +517,22 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         conn = null;
 
         conn = myds.getConnection();
+        callBack.setConnection(conn);
 
-        stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        stmt = callBack.getPreparedStatement();
+        assertNotNull(stmt);
         long l3HashCode = stmt.hashCode();
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
 
-        stmt = conn.prepareStatement("select * from dual");
-        assertTrue(null != stmt);
+        stmt = callBack.getPreparedStatement();
+        assertNotNull(stmt);
         long l4HashCode = stmt.hashCode();
         rset = stmt.executeQuery();
-        assertTrue(null != rset);
+        assertNotNull(rset);
         assertTrue(rset.next());
         rset.close();
         stmt.close();
@@ -515,6 +541,51 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         assertTrue(l3HashCode == l4HashCode);
         conn.close();
         conn = null;
+    }
+    public void testPoolPreparedStatements() throws Exception {
+        doTestPoolPreparedStatements(new PscbString());
+        doTestPoolPreparedStatements(new PscbStringIntInt());
+        doTestPoolPreparedStatements(new PscbStringInt());
+        doTestPoolPreparedStatements(new PscbStringIntArray());
+        doTestPoolPreparedStatements(new PscbStringStringArray());
+        doTestPoolPreparedStatements(new PscbStringIntIntInt());
+    }
 
+    // See DBCP-8
+    public void testChangePassword() throws Exception {
+        try {
+            ds.getConnection("foo", "bay");
+            fail("Should have generated SQLException");
+        } catch (SQLException expected) {
+        }
+        Connection con1 = ds.getConnection("foo", "bar");
+        Connection con2 = ds.getConnection("foo", "bar");
+        Connection con3 = ds.getConnection("foo", "bar");
+        con1.close();
+        con2.close();
+        TesterDriver.addUser("foo","bay"); // change the user/password setting
+        try {
+            Connection con4 = ds.getConnection("foo", "bay"); // new password
+            // Idle instances with old password should have been cleared
+            assertEquals("Should be no idle connections in the pool", 
+                   0, ((SharedPoolDataSource) ds).getNumIdle());
+            con4.close();
+            // Should be one idle instance with new pwd
+            assertEquals("Should be one idle connection in the pool", 
+                    1, ((SharedPoolDataSource) ds).getNumIdle());
+            try {
+                ds.getConnection("foo", "bar"); // old password
+                fail("Should have generated SQLException"); 
+            } catch (SQLException expected) {
+            }
+            Connection con5 = ds.getConnection("foo", "bay"); // take the idle one
+            con3.close(); // Return a connection with the old password
+            ds.getConnection("foo", "bay").close();  // will try bad returned connection and destroy it
+            assertEquals("Should be one idle connection in the pool", 
+                    1, ((SharedPoolDataSource) ds).getNumIdle());
+            con5.close();
+        } finally {
+            TesterDriver.addUser("foo","bar");
+        }
     }
 }
