@@ -17,10 +17,13 @@
 
 package org.apache.commons.jocl;
 
-import junit.framework.*;
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
+
 
 /**
- * @version $Revision: 479137 $ $Date: 2006-11-25 08:51:48 -0700 (Sat, 25 Nov 2006) $
+ * @version $Revision: 830125 $ $Date: 2009-10-27 06:36:25 -0400 (Tue, 27 Oct 2009) $
  */
 public class TestAll extends TestCase {
     public TestAll(String testName) {
